@@ -1,27 +1,13 @@
-/*=============================================================================
+/*
+    Copyright (C) 2013 Tom Bachmann
 
     This file is part of FLINT.
 
-    FLINT is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    FLINT is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with FLINT; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
-
-=============================================================================*/
-/******************************************************************************
-
-    Copyright (C) 2013 Tom Bachmann
-
-******************************************************************************/
+    FLINT is free software: you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 2.1 of the License, or
+    (at your option) any later version.  See <http://www.gnu.org/licenses/>.
+*/
 
 #ifndef FMPQ_MATXX_H
 #define FMPQ_MATXX_H FMPQ_MATXX_H
@@ -117,14 +103,14 @@ public:
     }
 
     static fmpq_matxx_expression randbits(slong rows, slong cols,
-            frandxx& state, mp_bitcnt_t bits)
+            frandxx& state, flint_bitcnt_t bits)
     {
         fmpq_matxx_expression res(rows, cols);
         res.set_randbits(state, bits);
         return res;
     }
     static fmpq_matxx_expression randtest(slong rows, slong cols,
-            frandxx& state, mp_bitcnt_t bits)
+            frandxx& state, flint_bitcnt_t bits)
     {
         fmpq_matxx_expression res(rows, cols);
         res.set_randtest(state, bits);
@@ -141,9 +127,9 @@ public:
     }
 
     // these only make sense with targets
-    void set_randbits(frandxx& state, mp_bitcnt_t bits)
+    void set_randbits(frandxx& state, flint_bitcnt_t bits)
         {fmpq_mat_randbits(_mat(), state._data(), bits);}
-    void set_randtest(frandxx& state, mp_bitcnt_t bits)
+    void set_randtest(frandxx& state, flint_bitcnt_t bits)
         {fmpq_mat_randtest(_mat(), state._data(), bits);}
     void set_hilbert_matrix()
         {fmpq_mat_hilbert_matrix(_mat());}
